'use strict'

/** All dialog wrappers */
export var wrappers = {}

/**
 * Map props definition to args.
 *
 * If the name of the first prop is one of the keys of the first argument,
 * that argument will be ignored.
 *
 * e.g. `makeDialog(component, 'title')({ title: 'some title' })`.
 * The `title` will be `'some title'`, not the object `{ title: 'some title' }`.
 * This will be less ambiguous.
 *
 * @param {string[]} props
 * @param {any[]} args
 */
function collectProps (props, args) {
  return props.reduce(function (propsData, prop, i) {
    if (
      (i !== 0 && args[i] !== undefined) ||
      typeof args[0] !== 'object' ||
      args[0][props[0]] === undefined
    ) {
      propsData[prop] = args[i]
    }

    return propsData
  }, {})
}

/** DialogsWrapper component */
export default {
  name: 'DialogsWrapper',
  props: {
    name: {
      type: String,
      default: 'default',
      validator: function (value) { return value; }
    },
    transitionName: String,
    appear: Boolean,
    appearActiveClass: String,
    appearClass: String,
    appearToClass: String,
    css: Boolean,
    duration: String,
    enterActiveClass: String,
    enterClass: String,
    enterToClass: String,
    leaveActiveClass: String,
    leaveClass: String,
    leaveToClass: String,
    moveClass: String,
    tag: String,
    type: String
  },
  data: function () { return ({
    /** An auto-increment id */
    id: 0,
    /** All dialogs to render. Dialog render options is stored here */
    dialogs: {}
  }); },
  computed: {
    dialogIds: function dialogIds () {
      return Object.keys(this.dialogs)
    }
  },
  created: function created () {
    if (process.env.NODE_ENV !== 'production') {
      if (wrappers[this.name]) {
        console.error(("[vue-modal-dialogs] The wrapper '" + (this.name) + "' is already exist. Please make sure that every wrapper has a unique name."))
      }
    }

    // Expose wrapper component
    if (!wrappers[this.name]) {
      wrappers[this.name] = this
    }
  },
  render: function render (createElement) {
    var this$1 = this;

    // Render the wrapper as transition-group
    return createElement(
      'transition-group',
      {
        props: Object.assign({},
          this.$attrs, this.$props,
          { name: this.transitionName }
        ),
        on: this.$listeners
      },
      this.dialogIds.map(function (dialogId) {
        var dialog = this$1.dialogs[dialogId]
        return createElement(dialog.component, {
          key: dialog.id,
          props: dialog.propsData,
          on: { 'vue-modal-dialogs:close': dialog.close }
        })
      })
    )
  },
  methods: {
    /**
     * Add a new dialog into this wrapper
     *
     * @private
     * @param {Object} options Dialog options created in the `makeDialog` function
     * @param {any[]} args Arguments from the dialog function
     */
    add: function add (options, args) {
      var id = this.id++
      var resolve

      // This promise will be resolved when 'close' function is called
      var promise = new Promise(function (res) { resolve = res })

      // Prepare the props of the dialog component
      var defaultPropsData = {
        dialogId: id,
        arguments: args
      }

      // If the first argument of the dialog function is an object,
      // use it as a part of the propsData
      var firstArgObject = typeof args[0] === 'object' ? args[0] : {}
      var propsData = Object.assign(
        {},
        defaultPropsData,
        firstArgObject,
        collectProps(options.props, args)
      )

      return this.pushDialog(Object.assign.apply(Object, [ { id: id, propsData: propsData, promise: promise, resolve: resolve } ].concat( options )))
    },

    /**
     * Add a dialog to `this.dialogs`
     *
     * @private
     * @param {Object} renderOptions Dialog render options generated in the `add` method
     * @returns {Promise} The promise
     */
    pushDialog: function pushDialog (renderOptions) {
      var this$1 = this;

      // Resolve previously created promise in 'add' method
      renderOptions.close = function (data) {
        renderOptions.resolve(data)
        return renderOptions.promise
      }

      // Remove the dialog after it is closed
      renderOptions.promise = renderOptions.promise.then(function (data) {
        this$1.$delete(this$1.dialogs, renderOptions.id)
        return data
      })

      // Inject 'close' function into `promise`
      renderOptions.promise.close = renderOptions.close

      // Use Object.freeze to prevent vue from observing renderOptions
      this.$set(this.dialogs, renderOptions.id, Object.freeze(renderOptions))

      return renderOptions.promise
    }
  }
}
